@@ -14,7 +14,8 @@ static int cert_callback(gnutls_session session,
 			 const gnutls_datum * req_ca_rdn, int nreqs)
 {
    char issuer_dn[256];
-   int len, i, ret;
+   int i, ret;
+   size_t len;
 
    /* Print the server's trusted CAs
     */
@@ -33,7 +34,7 @@ static int cert_callback(gnutls_session session,
       }
    }
 
-   /* Select a certificate from the client_certs and return it's
+   /* Select a certificate from the client_certs and return its
     * index.
     */
 
