@@ -44,7 +44,8 @@
 int gnutls_x509_extract_dn(const gnutls_datum * idn, gnutls_x509_dn * rdn)
 {
 	ASN1_TYPE dn = ASN1_TYPE_EMPTY;
-	int result, len;
+	int result;
+	size_t len;
 
 	if ((result =
 	     asn1_create_element(_gnutls_get_pkix(),
@@ -63,25 +64,25 @@ int gnutls_x509_extract_dn(const gnutls_datum * idn, gnutls_x509_dn * rdn)
 	memset( rdn, 0, sizeof(gnutls_x509_dn));
 
 	len = sizeof(rdn->country);
-	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_COUNTRY_NAME, 0, rdn->country, &len);
+	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_COUNTRY_NAME, 0, 0, rdn->country, &len);
 
 	len = sizeof(rdn->organization);
-	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_ORGANIZATION_NAME, 0, rdn->organization, &len);
+	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_ORGANIZATION_NAME, 0, 0, rdn->organization, &len);
 
 	len = sizeof(rdn->organizational_unit_name);
-	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0, rdn->organizational_unit_name, &len);
+	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0, 0, rdn->organizational_unit_name, &len);
 
 	len = sizeof(rdn->common_name);
-	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_COMMON_NAME, 0, rdn->common_name, &len);
+	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_COMMON_NAME, 0, 0, rdn->common_name, &len);
 
 	len = sizeof(rdn->locality_name);
-	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_LOCALITY_NAME, 0, rdn->locality_name, &len);
+	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_LOCALITY_NAME, 0, 0, rdn->locality_name, &len);
 
 	len = sizeof(rdn->state_or_province_name);
-	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME, 0, rdn->state_or_province_name, &len);
+	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME, 0, 0, rdn->state_or_province_name, &len);
 
 	len = sizeof(rdn->email);
-	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_PKCS9_EMAIL, 0, rdn->email, &len);
+	_gnutls_x509_parse_dn_oid( dn, "", GNUTLS_OID_PKCS9_EMAIL, 0, 0, rdn->email, &len);
 
 	asn1_delete_structure(&dn);
 
@@ -103,7 +104,8 @@ int gnutls_x509_extract_certificate_dn(const gnutls_datum * cert,
 					  gnutls_x509_dn * ret)
 {
 	gnutls_x509_crt xcert;
-	int len, result;
+	int result;
+	size_t len;
 	
 	result = gnutls_x509_crt_init( &xcert);
 	if (result < 0) return result;
@@ -115,31 +117,31 @@ int gnutls_x509_extract_certificate_dn(const gnutls_datum * cert,
 	}
 
 	len = sizeof( ret->country);
-	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_COUNTRY_NAME, 0,
+	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_COUNTRY_NAME, 0, 0,
 		ret->country, &len);
 
 	len = sizeof( ret->organization);
-	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_ORGANIZATION_NAME, 0,
+	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_ORGANIZATION_NAME, 0, 0,
 		ret->organization, &len);
 
 	len = sizeof( ret->organizational_unit_name);
-	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0,
+	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0, 0,
 		ret->organizational_unit_name, &len);
 
 	len = sizeof( ret->common_name);
-	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_COMMON_NAME, 0,
+	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_COMMON_NAME, 0, 0,
 		ret->common_name, &len);
 
 	len = sizeof( ret->locality_name);
-	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_LOCALITY_NAME, 0,
+	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_LOCALITY_NAME, 0, 0,
 		ret->locality_name, &len);
 
 	len = sizeof( ret->state_or_province_name);
-	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME, 0,
+	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME, 0, 0,
 		ret->state_or_province_name, &len);
 
 	len = sizeof( ret->email);
-	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_PKCS9_EMAIL, 0,
+	gnutls_x509_crt_get_dn_by_oid( xcert, GNUTLS_OID_PKCS9_EMAIL, 0, 0,
 		ret->email, &len);
 
 	gnutls_x509_crt_deinit( xcert);
@@ -162,8 +164,9 @@ int gnutls_x509_extract_certificate_issuer_dn(const gnutls_datum * cert,
 						 gnutls_x509_dn * ret)
 {
 	gnutls_x509_crt xcert;
-	int len, result;
-	
+	int result;
+	size_t len;
+
 	result = gnutls_x509_crt_init( &xcert);
 	if (result < 0) return result;
 	
@@ -174,31 +177,31 @@ int gnutls_x509_extract_certificate_issuer_dn(const gnutls_datum * cert,
 	}
 
 	len = sizeof( ret->country);
-	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_COUNTRY_NAME, 0,
+	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_COUNTRY_NAME, 0, 0,
 		ret->country, &len);
 
 	len = sizeof( ret->organization);
-	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_ORGANIZATION_NAME, 0,
+	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_ORGANIZATION_NAME, 0, 0,
 		ret->organization, &len);
 
 	len = sizeof( ret->organizational_unit_name);
-	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0,
+	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0, 0,
 		ret->organizational_unit_name, &len);
 
 	len = sizeof( ret->common_name);
-	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_COMMON_NAME, 0,
+	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_COMMON_NAME, 0, 0,
 		ret->common_name, &len);
 
 	len = sizeof( ret->locality_name);
-	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_LOCALITY_NAME, 0,
+	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_LOCALITY_NAME, 0, 0,
 		ret->locality_name, &len);
 
 	len = sizeof( ret->state_or_province_name);
-	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME, 0,
+	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME, 0, 0,
 		ret->state_or_province_name, &len);
 
 	len = sizeof( ret->email);
-	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_PKCS9_EMAIL, 0,
+	gnutls_x509_crt_get_issuer_dn_by_oid( xcert, GNUTLS_OID_PKCS9_EMAIL, 0, 0,
 		ret->email, &len);
 
 	gnutls_x509_crt_deinit( xcert);
@@ -232,6 +235,7 @@ int gnutls_x509_extract_certificate_subject_alt_name(const gnutls_datum * cert,
 {
 	gnutls_x509_crt xcert;
 	int result;
+	size_t size = *ret_size;
 	
 	result = gnutls_x509_crt_init( &xcert);
 	if (result < 0) return result;
@@ -242,7 +246,8 @@ int gnutls_x509_extract_certificate_subject_alt_name(const gnutls_datum * cert,
 		return result;
 	}
 	
-	result = gnutls_x509_crt_get_subject_alt_name( xcert, seq, ret, ret_size, NULL);
+	result = gnutls_x509_crt_get_subject_alt_name( xcert, seq, ret, &size, NULL);
+	*ret_size = size;
 	
 	gnutls_x509_crt_deinit( xcert);
 	
@@ -395,6 +400,7 @@ int gnutls_x509_extract_certificate_version(const gnutls_datum * cert)
 int gnutls_x509_extract_certificate_serial(const gnutls_datum * cert, char* result, int* result_size)
 {
 	gnutls_x509_crt xcert;
+	size_t size = *result_size;
 	int ret;
 
 	ret = gnutls_x509_crt_init( &xcert);
@@ -406,7 +412,8 @@ int gnutls_x509_extract_certificate_serial(const gnutls_datum * cert, char* resu
 		return ret;
 	}
 	
-	ret = gnutls_x509_crt_get_serial( xcert, result, result_size);
+	ret = gnutls_x509_crt_get_serial( xcert, result, &size);
+	*result_size = size;
 	
 	gnutls_x509_crt_deinit( xcert);
 	
@@ -733,6 +740,7 @@ int gnutls_x509_pkcs7_extract_certificate(const gnutls_datum * pkcs7_struct, int
 {
 	gnutls_pkcs7 pkcs7;
 	int result;
+	size_t size = *certificate_size;
 
 	result = gnutls_pkcs7_init( &pkcs7);
 	if (result < 0) return result;
@@ -743,8 +751,9 @@ int gnutls_x509_pkcs7_extract_certificate(const gnutls_datum * pkcs7_struct, int
 		return result;
 	}
 	
-	result = gnutls_pkcs7_get_crt_raw( pkcs7, indx, certificate, certificate_size);
-	
+	result = gnutls_pkcs7_get_crt_raw( pkcs7, indx, certificate, &size);
+	*certificate_size = size;
+
 	gnutls_pkcs7_deinit( pkcs7);
 	
 	return result;
