@@ -32,6 +32,7 @@
 #include "auth_srp.h"
 #include <gnutls_str.h>
 #include <gnutls_datum.h>
+#include <gnutls_alert.h>
 
 int _gnutls_gen_srp_server_kx(gnutls_session, opaque **);
 int _gnutls_gen_srp_client_kx(gnutls_session, opaque **);
@@ -57,19 +58,19 @@ const MOD_AUTH_STRUCT srp_auth_struct = {
 };
 
 
-#define _b state->key->b
-#define B state->key->B
-#define _a state->key->a
-#define A state->key->A
-#define N state->key->client_p
-#define G state->key->client_g
-#define V state->key->x
-#define S state->key->KEY
+#define _b session->key->b
+#define B session->key->B
+#define _a session->key->a
+#define A session->key->A
+#define N session->key->client_p
+#define G session->key->client_g
+#define V session->key->x
+#define S session->key->KEY
 
 /* Send the first key exchange message ( g, n, s) and append the verifier algorithm number 
  * Data is allocated by the caller, and should have data_size size.
  */
-int _gnutls_gen_srp_server_kx(gnutls_session state, opaque ** data)
+int _gnutls_gen_srp_server_kx(gnutls_session session, opaque ** data)
 {
 	int ret;
 	uint8 *data_n, *data_s;
@@ -82,32 +83,24 @@ int _gnutls_gen_srp_server_kx(gnutls_session state, opaque ** data)
 	char buf[64];
 	uint8 *data_b;
 
-	if (state->security_parameters.extensions.srp_username[0] == 0) {
-		/* The peer didn't send a valid SRP extension with the
-		 * SRP username.
-		 */
-		gnutls_assert();
-		return GNUTLS_E_EMPTY_SRP_USERNAME;
-	}
-
-	if ( (ret=_gnutls_auth_info_set( state, GNUTLS_CRD_SRP, sizeof( SRP_SERVER_AUTH_INFO_INT), 1)) < 0) {
+	if ( (ret=_gnutls_auth_info_set( session, GNUTLS_CRD_SRP, sizeof( SRP_SERVER_AUTH_INFO_INT), 1)) < 0) {
 		gnutls_assert();
 		return ret;
 	}
 
-	info = _gnutls_get_auth_info( state);
+	info = _gnutls_get_auth_info( session);
 	username = info->username;
 
-	_gnutls_str_cpy( username, MAX_SRP_USERNAME, state->security_parameters.extensions.srp_username);
+	_gnutls_str_cpy( username, MAX_SRP_USERNAME, session->security_parameters.extensions.srp_username);
 
-	ret = _gnutls_srp_pwd_read_entry( state, username, &pwd_entry);
+	ret = _gnutls_srp_pwd_read_entry( session, username, &pwd_entry);
 
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
 	}
 
-	/* copy from pwd_entry to local variables (actually in state) */
+	/* copy from pwd_entry to local variables (actually in session) */
 	if (_gnutls_mpi_scan( &G, pwd_entry->g.data, &pwd_entry->g.size) < 0) {
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
@@ -185,7 +178,7 @@ int _gnutls_gen_srp_server_kx(gnutls_session state, opaque ** data)
 }
 
 /* return A = g^a % N */
-int _gnutls_gen_srp_client_kx(gnutls_session state, opaque ** data)
+int _gnutls_gen_srp_client_kx(gnutls_session session, opaque ** data)
 {
 	size_t n_a;
 	int ret;
@@ -194,15 +187,20 @@ int _gnutls_gen_srp_client_kx(gnutls_session state, opaque ** data)
 	char buf[64];
 	char *password;
 	const gnutls_srp_client_credentials cred =
-	    _gnutls_get_cred(state->key, GNUTLS_CRD_SRP, NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_SRP, NULL);
 
 	if (cred == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 	
-	username = cred->username;
-	password = cred->password;
+	if (session->internals.srp_username == NULL) {
+		username = cred->username;
+		password = cred->password;
+	} else {
+		username = session->internals.srp_username;
+		password = session->internals.srp_password;
+	}
 
 	if (username == NULL || password == NULL) {
 		gnutls_assert();
@@ -226,16 +224,16 @@ int _gnutls_gen_srp_client_kx(gnutls_session state, opaque ** data)
 	 */
 
 	/* calculate u */
-	state->key->u = _gnutls_calc_srp_u(A, B);
-	if ( state->key->u == NULL) {
+	session->key->u = _gnutls_calc_srp_u(A, B);
+	if ( session->key->u == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	_gnutls_dump_mpi( "SRP U: ", state->key->u);
+	_gnutls_dump_mpi( "SRP U: ", session->key->u);
 
 	/* S = (B - g^x) ^ (a + u * x) % N */
-	S = _gnutls_calc_srp_S2( B, G, state->key->x, _a, state->key->u, N);
+	S = _gnutls_calc_srp_S2( B, G, session->key->x, _a, session->key->u, N);
 	if (S==NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
@@ -245,10 +243,10 @@ int _gnutls_gen_srp_client_kx(gnutls_session state, opaque ** data)
 	
 	_gnutls_mpi_release(&_b);
 	_gnutls_mpi_release(&V);
-	_gnutls_mpi_release(&state->key->u);
+	_gnutls_mpi_release(&session->key->u);
 	_gnutls_mpi_release(&B);
 
-	ret = _gnutls_generate_session_key( state->key);
+	ret = _gnutls_generate_session_key( session->key);
 	_gnutls_mpi_release(&S);
 
 	if (ret < 0)
@@ -281,8 +279,8 @@ int _gnutls_gen_srp_client_kx(gnutls_session state, opaque ** data)
 }
 
 
-/* just read A and put it to state */
-int _gnutls_proc_srp_client_kx(gnutls_session state, opaque * data, size_t _data_size)
+/* just read A and put it to session */
+int _gnutls_proc_srp_client_kx(gnutls_session session, opaque * data, size_t _data_size)
 {
 	size_t _n_A;
 	ssize_t data_size = _data_size;
@@ -303,17 +301,17 @@ int _gnutls_proc_srp_client_kx(gnutls_session state, opaque * data, size_t _data
 	/* Start the SRP calculations.
 	 * - Calculate u 
 	 */
-	state->key->u = _gnutls_calc_srp_u(A, B);
-	if (state->key->u==NULL) {
+	session->key->u = _gnutls_calc_srp_u(A, B);
+	if (session->key->u==NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
-	_gnutls_dump_mpi( "SRP U: ", state->key->u);
+	_gnutls_dump_mpi( "SRP U: ", session->key->u);
 
 	/* S = (A * v^u) ^ b % N 
 	 */
-	S = _gnutls_calc_srp_S1( A, _b, state->key->u, V, N);
+	S = _gnutls_calc_srp_S1( A, _b, session->key->u, V, N);
 	if ( S==NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
@@ -324,10 +322,10 @@ int _gnutls_proc_srp_client_kx(gnutls_session state, opaque * data, size_t _data
 	_gnutls_mpi_release(&A);
 	_gnutls_mpi_release(&_b);
 	_gnutls_mpi_release(&V);
-	_gnutls_mpi_release(&state->key->u);
+	_gnutls_mpi_release(&session->key->u);
 	_gnutls_mpi_release(&B);
 
-	ret = _gnutls_generate_session_key( state->key);
+	ret = _gnutls_generate_session_key( session->key);
 	_gnutls_mpi_release( &S);
 
 	if (ret < 0) {
@@ -384,6 +382,11 @@ static const unsigned char srp_params_1024[] = {
 
 static const unsigned char srp_generator = 0x02;
 
+const gnutls_datum gnutls_srp_1024_group_prime = {
+	(void*)srp_params_1024, sizeof(srp_params_1024) };
+const gnutls_datum gnutls_srp_1024_group_generator = {
+	(void*)&srp_generator, sizeof(srp_generator) };
+
 static const unsigned char srp_params_1536[] = {
  0x9D, 0xEF, 0x3C, 0xAF, 0xB9, 0x39, 0x27, 0x7A, 0xB1,
  0xF1, 0x2A, 0x86, 0x17, 0xA4, 0x7B, 0xBB, 0xDB, 0xA5,
@@ -409,6 +412,11 @@ static const unsigned char srp_params_1536[] = {
  0x35, 0xF9, 0xBB
 };
 
+const gnutls_datum gnutls_srp_1536_group_prime = {
+	(void*)srp_params_1536, sizeof(srp_params_1536) };
+const gnutls_datum gnutls_srp_1536_group_generator = {
+	(void*)&srp_generator, sizeof(srp_generator) };
+
 static const unsigned char srp_params_2048[] = {
  0xAC, 0x6B, 0xDB, 0x41, 0x32, 0x4A, 0x9A, 0x9B, 0xF1,
  0x66, 0xDE, 0x5E, 0x13, 0x89, 0x58, 0x2F, 0xAF, 0x72,
@@ -441,6 +449,12 @@ static const unsigned char srp_params_2048[] = {
  0x9E, 0x4A, 0xFF, 0x73
 };
 
+const gnutls_datum gnutls_srp_2048_group_prime = {
+	(void*)srp_params_2048, sizeof(srp_params_2048) };
+const gnutls_datum gnutls_srp_2048_group_generator = {
+	(void*)&srp_generator, sizeof(srp_generator) };
+
+
 /* Check if G and N are parameters from the SRP draft.
  */
 static int check_g_n( const opaque* g, size_t n_g, 
@@ -495,7 +509,7 @@ static int group_check_g_n( GNUTLS_MPI g, GNUTLS_MPI n)
 
 /* receive the key exchange message ( n, g, s, B) 
  */
-int _gnutls_proc_srp_server_kx(gnutls_session state, opaque * data, size_t _data_size)
+int _gnutls_proc_srp_server_kx(gnutls_session session, opaque * data, size_t _data_size)
 {
 	uint8 n_s;
 	uint16 n_g, n_n, n_b;
@@ -510,15 +524,20 @@ int _gnutls_proc_srp_server_kx(gnutls_session state, opaque * data, size_t _data
 	ssize_t data_size = _data_size;
 
 	const gnutls_srp_client_credentials cred =
-	    _gnutls_get_cred(state->key, GNUTLS_CRD_SRP, NULL);
+	    _gnutls_get_cred(session->key, GNUTLS_CRD_SRP, NULL);
 
 	if (cred == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 	}
 	
-	username = cred->username;
-	password = cred->password;
+	if (session->internals.srp_username == NULL) {
+		username = cred->username;
+		password = cred->password;
+	} else {
+		username = session->internals.srp_username;
+		password = session->internals.srp_password;
+	}
 
 	if (username == NULL || password == NULL) {
 		gnutls_assert();
@@ -616,13 +635,15 @@ int _gnutls_proc_srp_server_kx(gnutls_session state, opaque * data, size_t _data
 		return ret;
 	}
 
-	if (_gnutls_mpi_scan(&state->key->x, hd, &_n_g) != 0) {
+	if (_gnutls_mpi_scan(&session->key->x, hd, &_n_g) != 0) {
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
 	
 
-	return 0;
+    return i; /* return the processed data
+               * needed in auth_srp_rsa.
+               */
 }
 
 #endif /* ENABLE_SRP */
